/* eslint no-param-reassign: 0 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.nodeIdToCursor = nodeIdToCursor;
exports.cursorToNodeId = cursorToNodeId;
exports.pkToCursor = pkToCursor;
exports.cursorToPk = cursorToPk;
exports.dataToEdge = dataToEdge;
exports.pkToOffset = pkToOffset;
exports.cursorToOffset = cursorToOffset;
exports.assertConnectionArgs = assertConnectionArgs;
exports.applyCursorsToQuery = applyCursorsToQuery;
exports.applyLimitsToQuery = applyLimitsToQuery;
exports.fieldFromConnectionType = fieldFromConnectionType;
exports.connectionField = connectionField;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _rethinkdb = require('rethinkdb');

var _rethinkdb2 = _interopRequireDefault(_rethinkdb);

var _graphqlRelay = require('graphql-relay');

var _graphqlError = require('graphql/error');

var _base64 = require('./base64');

var PREFIX = 'arrayconnection:';

function nodeIdToCursor(nodeId) {
  return (0, _base64.base64)(PREFIX + nodeId);
}

function cursorToNodeId(cursor) {
  return (0, _base64.unbase64)(cursor).substring(PREFIX.length);
}

function pkToCursor(tableName, pk) {
  return nodeIdToCursor((0, _graphqlRelay.toGlobalId)(tableName, pk));
}

function cursorToPk(cursor) {
  var _fromGlobalId = (0, _graphqlRelay.fromGlobalId)(cursorToNodeId(cursor));

  var resourceId = _fromGlobalId.id;

  return resourceId;
}

function dataToEdge(table, data) {
  var pk = data[table.pk];
  return {
    cursor: pkToCursor(table.tableName, pk),
    node: data
  };
}

function pkToOffset(query, pk) {
  var offset = query.offsetsOf(function (row) {
    return row('id').eq(pk);
  }).nth(0);
  return offset;
}

function cursorToOffset(query, cursor) {
  return pkToOffset(query, cursorToPk(cursor));
}

function assertConnectionArgs(_ref) {
  var first = _ref.first;
  var last = _ref.last;

  if (!_lodash2['default'].isNumber(first) && !_lodash2['default'].isNumber(last)) {
    throw new _graphqlError.GraphQLError('\'first\' or \'last\' should be given.');
  }
  if (_lodash2['default'].some([first, last], function (amount) {
    return _lodash2['default'].isNumber(amount) && amount <= 0;
  })) {
    throw new _graphqlError.GraphQLError('\'first\' and \'last\' must more than 0');
  }
}

// https://facebook.github.io/relay/graphql/connections.htm#ApplyCursorsToEdges()

function applyCursorsToQuery(query, _ref2) {
  var after = _ref2.after;
  var before = _ref2.before;

  return query.slice(after ? pkToOffset(query, cursorToPk(after)).add(1) : 0, before ? pkToOffset(query, cursorToPk(before)) : undefined);
}

// https://facebook.github.io/relay/graphql/connections.htm#EdgesToReturn()

function applyLimitsToQuery(query, _ref3) {
  var first = _ref3.first;
  var last = _ref3.last;

  if (_lodash2['default'].isNumber(first)) {
    query = query.limit(first);
  }

  if (_lodash2['default'].isNumber(last)) {
    query = query.coerceTo('array').slice(last * -1);
  }

  return query;
}

function fieldFromConnectionType(_ref4) {
  var _this = this;

  var connectionType = _ref4.connectionType;
  var table = _ref4.table;
  var connect = _ref4.connect;
  var optionArgs = _ref4.args;
  var _ref4$getQuery = _ref4.getQuery;
  var getQuery = _ref4$getQuery === undefined ? function () {
    return (/* root, args, context, info */table.query().orderBy({ index: _rethinkdb2['default'].desc('createdAt') })
    );
  } : _ref4$getQuery;
  var _ref4$runQuery = _ref4.runQuery;
  var runQuery = _ref4$runQuery === undefined ? function callee$1$0(query) {
    var connection, result;
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          (0, _assert2['default'])(_lodash2['default'].isFunction(connect), 'connect should be function');
          context$2$0.next = 3;
          return regeneratorRuntime.awrap(connect());

        case 3:
          connection = context$2$0.sent;
          context$2$0.next = 6;
          return regeneratorRuntime.awrap(query.run(connection));

        case 6:
          result = context$2$0.sent;

          connection.close();
          return context$2$0.abrupt('return', result);

        case 9:
        case 'end':
          return context$2$0.stop();
      }
    }, null, _this);
  } : _ref4$runQuery;
  var _ref4$afterQuery = _ref4.afterQuery;
  var afterQuery = _ref4$afterQuery === undefined ? function (query) {
    return query;
  } : _ref4$afterQuery;
  return (function () {
    var _this2 = this;

    return {
      type: connectionType,
      args: _extends({}, optionArgs, _graphqlRelay.connectionArgs),
      resolve: function resolve(root, args, context, info) {
        var after, before, first, last, query, edgesLength, rows, edges, firstEdge, lastEdge;
        return regeneratorRuntime.async(function resolve$(context$3$0) {
          while (1) switch (context$3$0.prev = context$3$0.next) {
            case 0:
              after = args.after;
              before = args.before;
              first = args.first;
              last = args.last;

              assertConnectionArgs({ first: first, last: last });

              context$3$0.next = 7;
              return regeneratorRuntime.awrap(getQuery(root, args, context, info));

            case 7:
              query = context$3$0.sent;

              query = applyCursorsToQuery(query, { after: after, before: before });
              context$3$0.next = 11;
              return regeneratorRuntime.awrap(runQuery(query.count()));

            case 11:
              edgesLength = context$3$0.sent;

              query = applyLimitsToQuery(query, { first: first, last: last });
              query = afterQuery(query, root, args, context, info);
              query = query.coerceTo('array');
              context$3$0.next = 17;
              return regeneratorRuntime.awrap(runQuery(query));

            case 17:
              rows = context$3$0.sent;
              edges = rows.map(function (row) {
                return dataToEdge(table, row);
              });
              firstEdge = _lodash2['default'].head(edges);
              lastEdge = _lodash2['default'].last(edges);
              return context$3$0.abrupt('return', {
                edges: edges,
                pageInfo: {
                  startCursor: firstEdge ? firstEdge.cursor : null,
                  endCursor: lastEdge ? lastEdge.cursor : null,
                  hasPreviousPage: last !== undefined ? edgesLength > last : false,
                  hasNextPage: first !== undefined ? edgesLength > first : false
                }
              });

            case 22:
            case 'end':
              return context$3$0.stop();
          }
        }, null, _this2);
      }
    };
  })();
}

function connectionField() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var graphQLType = options.graphQLType;
  var _options$name = options.name;
  var name = _options$name === undefined ? graphQLType.name : _options$name;

  var _connectionDefinitions = (0, _graphqlRelay.connectionDefinitions)({
    nodeType: graphQLType,
    name: name
  });

  var connectionType = _connectionDefinitions.connectionType;

  return fieldFromConnectionType(_extends({}, options, {
    connectionType: connectionType
  }));
}