'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.getGraphQLFieldsFromTable = getGraphQLFieldsFromTable;
exports.getGraphQLfieldsFromSchema = getGraphQLfieldsFromSchema;
exports.joiToGraphQLJoiType = joiToGraphQLJoiType;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _graphql = require('graphql');

var _graphqlRelay = require('graphql-relay');

var _type = require('./type');

function isJoi(schema) {
  return _lodash2['default'].has(schema, 'isJoi');
}

function isJoiCollection(schema) {
  return _lodash2['default'].isObject(schema) && !isJoi(schema);
}

function getGraphQLFieldsFromTable(table) {
  var schema = table.getSchema();

  return _extends({}, getGraphQLfieldsFromSchema(_lodash2['default'].omit(schema, table.pk)), _defineProperty({}, table.pk, (0, _graphqlRelay.globalIdField)(table.tableName)));
}

function getGraphQLfieldsFromSchema(schema, key) {
  if (isJoiCollection(schema)) {
    return _lodash2['default'].mapValues(schema, function (fieldSchema, fieldKey) {
      return getGraphQLfieldsFromSchema(fieldSchema, fieldKey);
    });
  }

  var GraphQLType = undefined;
  var type = schema._type;
  var description = schema._description;
  var flags = schema._flags;
  var tests = schema._tests;
  var inner = schema._inner;
  var valids = schema._valids;
  var meta = schema._meta;
  var unit = schema._unit;

  var _$reduce = _lodash2['default'].reduce(meta, function (result, value) {
    return _lodash2['default'].assignIn(result, value);
  }, {});

  var GraphQLField = _$reduce.GraphQLField;
  var metaGraphQLType = _$reduce.GraphQLType;

  if (!_lodash2['default'].isUndefined(GraphQLField)) return GraphQLField;

  switch (type) {
    case 'object':
      var name = unit || key;
      _assert2['default'].equal(_lodash2['default'].isEmpty(name), false, 'object must provide by key or joi.unit()');
      GraphQLType = new _graphql.GraphQLObjectType({
        name: name,
        fields: _lodash2['default'].reduce(inner.children, function (memo, child) {
          return _extends({}, memo, _defineProperty({}, child.key, getGraphQLfieldsFromSchema(child.schema, child.key)));
        }, {})
      });
      break;
    case 'array':
      _assert2['default'].equal(inner.items.length, 1, 'array shoud only have one type');

      var _getGraphQLfieldsFromSchema = getGraphQLfieldsFromSchema(inner.items[0]),
          InnerGraphQLType = _getGraphQLfieldsFromSchema.type;

      GraphQLType = new _graphql.GraphQLList(InnerGraphQLType);
      break;
    case 'boolean':
      GraphQLType = _graphql.GraphQLBoolean;
      break;
    case 'number':
      GraphQLType = _lodash2['default'].find(tests, { name: 'integer' }) ? _graphql.GraphQLInt : _graphql.GraphQLFloat;
      break;
    case 'date':
      GraphQLType = _type.GraphQLDateType;
      break;
    case 'string':
    default:
      GraphQLType = _graphql.GraphQLString;
      break;
  }

  if (flags.allowOnly) {
    _assert2['default'].equal(_lodash2['default'].isEmpty(valids._set), false, 'enum should have at least 1 item.');
    GraphQLType = new _graphql.GraphQLEnumType({
      name: key,
      values: _lodash2['default'].reduce(valids._set, function (result, value) {
        result[value] = {
          value: value
        };
        return result;
      }, {})
    });
  }

  if (!_lodash2['default'].isUndefined(metaGraphQLType)) {
    GraphQLType = metaGraphQLType;
  }

  if (flags.presence === 'required') {
    GraphQLType = new _graphql.GraphQLNonNull(GraphQLType);
  }

  return _lodash2['default'].omitBy({ type: GraphQLType, description: description }, _lodash2['default'].isEmpty);
}

function joiToGraphQLJoiType(schema, name) {
  if (isJoiCollection(schema)) {
    return _lodash2['default'].mapValues(schema, function (fieldSchema, fieldKey) {
      return joiToGraphQLJoiType(fieldSchema, fieldKey);
    });
  }

  var graphQLJoiType = new _type.GraphQLJoiType({ name: name, schema: schema });

  return graphQLJoiType.schema;
}